/* Autogenerated, do not edit */
#ifndef __CR_VERSION_H__
#define __CR_VERSION_H__
#define CRIU_VERSION "3.13"
#define CRIU_VERSION_MAJOR  3
#define CRIU_VERSION_MINOR  13
#define CRIU_GITID "0"
#endif /* __CR_VERSION_H__ */
